"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const shelljs_1 = __importDefault(require("shelljs"));
const util_1 = require("util");
const ora_1 = __importDefault(require("ora"));
const inquirer_1 = __importDefault(require("inquirer"));
const child_process_1 = require("child_process");
const execWithPromise = util_1.promisify(child_process_1.exec);
const cloneRoot = async (rootDir) => {
    const spinner = ora_1.default(`${rootDir} is cloning`);
    spinner.start();
    await execWithPromise(`git clone ${rootDir}.git`);
    spinner.succeed();
};
const checkDirExisted = async (dir) => {
    if (fs_1.default.existsSync(dir)) {
        const answers = await inquirer_1.default.prompt([{
                type: 'confirm',
                message: `${dir} folder already existed whether delete?`,
                name: 'delete',
                default: 'Yes'
            }]);
        if (answers.delete) {
            shelljs_1.default.rm('-rf', dir);
            console.log(`${dir} folder delete succeed`);
        }
        else
            process.exit();
    }
};
const deepCloneDirectory = async (rootDir, distDirName, branch) => {
    const spinner = ora_1.default(`${distDirName} is cloning`);
    spinner.start();
    await execWithPromise(`git init && git config core.sparsecheckout true`); // 设置允许克隆子目录
    await execWithPromise(`echo ${distDirName} > .git/info/sparse-checkout`); // clone 指定文件夹
    await execWithPromise(`git remote add origin ${rootDir}.git`);
    await execWithPromise(`git pull origin ${branch}`);
    spinner.succeed();
};
const httpToSSH = (rootDir, dir) => {
    const isGitLab = /gitlab/i.test(dir);
    let res;
    // gitlab下http地址转ssh
    res = rootDir[rootDir.length - 1] === '/' ? rootDir.slice(0, rootDir.length - 1) : rootDir;
    if (!isGitLab)
        return res;
    res = res.replace(/https?:\/\//, '');
    const index = res.indexOf('/');
    res = res.substring(0, index) + ':' + res.substring(index + 1);
    return 'git@' + res;
};
exports.httpToSSH = httpToSSH;
const dclone = async (dir) => {
    if (!dir) {
        console.error('please enter the name of directory');
        return;
    }
    let [rootDir, ...distDirArr] = dir.split('tree');
    rootDir = httpToSSH(rootDir, dir);
    if (distDirArr.length === 0) {
        await cloneRoot(rootDir); // 说明clone的是根目录
        return;
    }
    const distDir = distDirArr.length > 1 ? distDirArr.join('tree') : distDirArr.join(''); // 修复url中存在多个tree的情况，只以第一个作为分割点
    const [, branch, ...distDirNameArr] = distDir.split('/');
    await checkDirExisted(distDirNameArr[0]);
    const distDirName = distDirNameArr.join('/');
    await deepCloneDirectory(rootDir, distDirName, branch); // 深度clone子目录
    shelljs_1.default.rm('-rf', '.git');
    process.exit();
};
exports.dclone = dclone;
